@@ -316,6 +316,53 @@ else:
         https_request = http_request
         https_response = http_response
 
+    try:
+        import robotparser
+    except ImportError:
+        pass
+    else:
+        class RobotExclusionError(urllib2.HTTPError):
+            def __init__(self, request, *args):
+                apply(urllib2.HTTPError.__init__, (self,)+args)
+                self.request = request
+
+        class HTTPRobotRulesProcessor(BaseProcessor):
+            # before redirections and response debugging, after everything else
+            processor_order = 800
+
+            try:
+                from httplib import HTTPMessage
+            except:
+                from mimetools import Message
+                http_response_class = Message
+            else:
+                http_response_class = HTTPMessage
+
+            def __init__(self, rfp_class=robotparser.RobotFileParser):
+                self.rfp_class = rfp_class
+                self.rfp = None
+                self._host = None
+            def http_request(self, request):
+                host = request.get_host()
+                if host != self._host:
+                    self.rfp = self.rfp_class()
+                    self.rfp.set_url("http://"+host+"/robots.txt")
+                    self.rfp.read()
+                    self._host = host
+
+                ua = request.get_header("User-agent", "")
+                if self.rfp.can_fetch(ua, request.get_full_url()):
+                    return request
+                else:
+                    msg = "request disallowed by robots.txt"
+                    raise RobotExclusionError(
+                        request,
+                        request.get_full_url(),
+                        403, msg,
+                        self.http_response_class(StringIO()), StringIO(msg))
+
+            https_request = http_request
+
     class HTTPRefererProcessor(BaseProcessor):
         """Add Referer header to requests.
 
@@ -394,7 +441,6 @@ else:
                             if pause != 0 and self.honor_time:
                                 time.sleep(pause)
                             newurl = newurl_spec[i+1:]
-                            # fake a 302 response
                             hdrs["location"] = newurl
                             response = self.parent.error(
                                 'http', request, response,
