@@ -294,6 +294,23 @@ def sanepathname2url(path):
     # XXX don't ask me about the mac...
     return urlpath
 
+class MockRobotFileParserClass:
+    def __init__(self):
+        self.calls = []
+        self._can_fetch = True
+    def clear(self):
+        self.calls = []
+    def __call__(self):
+        self.calls.append("__call__")
+        return self
+    def set_url(self, url):
+        self.calls.append(("set_url", url))
+    def read(self):
+        self.calls.append("read")
+    def can_fetch(self, ua, url):
+        self.calls.append(("can_fetch", ua, url))
+        return self._can_fetch
+
 class HandlerTests(unittest.TestCase):
 
     if hasattr(sys, "version_info") and sys.version_info > (2, 1, 3, "final", 0):
@@ -619,6 +636,68 @@ class HandlerTests(unittest.TestCase):
         self.assert_(o.proto == "http")  # o.error called
         self.assert_(o.args == (req, r, 201, "Created", {}))
 
+    def test_robots(self):
+        # XXX useragent
+        try:
+            import robotparser
+        except ImportError:
+            return  # skip test
+        else:
+            from ClientCookie import HTTPRobotRulesProcessor
+        rfpc = MockRobotFileParserClass()
+        h = HTTPRobotRulesProcessor(rfpc)
+
+        url = "http://example.com:80/foo/bar.html"
+        req = Request(url)
+        # first time: initialise and set up robots.txt parser before checking
+        #  whether OK to fetch URL
+        h.http_request(req)
+        self.assert_(rfpc.calls == [
+            "__call__",
+            ("set_url", "http://example.com:80/robots.txt"),
+            "read",
+            ("can_fetch", "", url),
+            ])
+        # second time: just use existing parser
+        rfpc.clear()
+        req = Request(url)
+        h.http_request(req)
+        self.assert_(rfpc.calls == [
+            ("can_fetch", "", url),
+            ])
+        # different URL on same server: same again
+        rfpc.clear()
+        url = "http://example.com:80/blah.html"
+        req = Request(url)
+        h.http_request(req)
+        self.assert_(rfpc.calls == [
+            ("can_fetch", "", url),
+            ])
+        # disallowed URL
+        rfpc.clear()
+        rfpc._can_fetch = False
+        url = "http://example.com:80/rhubarb.html"
+        req = Request(url)
+        try:
+            h.http_request(req)
+        except urllib2.HTTPError, e:
+            self.assert_(e.request == req)
+            self.assert_(e.code == 403)
+        # new host: reload robots.txt (even though the host and port are
+        #  unchanged, we treat this as a new host because
+        #  "example.com" != "example.com:80")
+        rfpc.clear()
+        rfpc._can_fetch = True
+        url = "http://example.com/rhubarb.html"
+        req = Request(url)
+        h.http_request(req)
+        self.assert_(rfpc.calls == [
+            "__call__",
+            ("set_url", "http://example.com/robots.txt"),
+            "read",
+            ("can_fetch", "", url),
+            ])
+
     def test_cookies(self):
         cj = MockCookieJar()
         h = HTTPCookieProcessor(cj)
